###################################################
# SPER workshop on propensity score methods
# R code for performing PS matching analysis using
# NMES (National Medical Expenditures Survey) data
# Author: Brian Lee (bklee@drexel.edu)
# Date: 6/17/2013
###################################################

#~~~ Libraries ~~~#
# need to install first, if you haven't already installed, 
# just type: install.packages("package_name"), e.g. install.packages("twang")
library(twang) # for boosted CART
library(MatchIt) # for PS matching

#~~~ Initial data setup ~~~#

    # set working directory (change to your own path)
    setwd("d:/conferences/2013 sper/")
    
    # read in data
    a <- read.table("nmes.txt", header=T,sep="\t", na.string=".")

    # for didactic purposes, focus only on observations with complete data on variables we want
    # remove observations with missing data
    a <- a[!is.na(a$eversmk) & !is.na(a$POVSTALB) & !is.na(a$marital) & !is.na(a$beltuse), ]

    # create limited subset of covariates
    a <- a[, c("eversmk", "lc5", "LASTAGE", "MALE", "RACE3", "beltuse", "educate", "marital", "SREGION", "POVSTALB")]

    # make sure factor variables are factors
    a$marital <- as.factor(a$marital)
    a$POVSTALB <- as.factor(a$POVSTALB)
    a$educate <- as.factor(a$educate)
    a$beltuse <- as.factor(a$beltuse)
    a$RACE3 <- as.factor(a$RACE3)
    a$SREGION <- as.factor(a$SREGION)

#~~~ Propensity score model ~~~#
    # Model 1: a simple logistic regression PS model predicting eversmoke from covariates
    ps.model1 <- glm(eversmk ~ LASTAGE + MALE + educate + beltuse + POVSTALB + marital + RACE3 + SREGION, 
                    data=a, family=binomial(link="logit"))
    summary(ps.model1)
    
    # estimates odds of eversmoke, then convert to probability (aka the PS)
    odds1 <- exp(predict(ps.model1, a))
    a$PS <- odds1/(1+odds1) # the logistic regression estimated PS

    # be careful about model misspecification    
    # E.g.: is age really linearly associated with log odds of eversmoking?    
    # Plot age by percent of eversmoking to get a clue
    table(a$eversmk, a$LASTAGE) -> temp
    pct.eversmk <- 100*(temp[2, ] / (temp[2,] + temp[1,]))
    plot(40:94, pct.eversmk, xlab="Age", ylab="% ever smokers", pch=".", cex=7, cex.lab=1.5, cex.axis=1.5)

    # Model 2: Non-parametric PS model using boosted CART (Classification and regression tree) 
    # See Lee et al., 2010:
    # Caution: will take a few minutes to crunch, results will slightly differ every time due to machine learning randomness
    # For real purposes, suggest n.trees=10000 (number of trees) for better balance, but will take 5 times longer to run than n.trees=2000
    ps.model2 <- ps(eversmk ~ LASTAGE + MALE + educate + beltuse + POVSTALB + marital + RACE3 + SREGION, data=a, n.trees=2000)
    a$PS2 <- ps.model2$ps[, 1] # the boosted CART estimated PS

#~~~ PS matching using MatchIt ~~~#
    # MatchIt takes the PS model formula and estimates the PS for you - 
    # but we don't need it since we already estimated 2 different sets of pscores above
    # therefore the covariates there are non-functional, but still put in those covariates so you can check balance on them
    
    # Matching attempt #1
    # Nearest neighbor using logistic regression estimated PS, 1:1
    nn1 <- matchit(eversmk ~ LASTAGE + MALE + educate + beltuse + POVSTALB + marital + RACE3 + SREGION, 
                    data=a, distance=a$PS, method="nearest")
    nn1.data <- match.data(nn1)
    summary(nn1, standardize=T)

    # Matching attempt #2
    # Nearest neighbor 1:1 matching using boosted CART estimated PS, 1:1,
    # discarding treated or controls beyond range of PS overlap, exact matching on sex
    nn2 <- matchit(eversmk ~ LASTAGE + MALE + educate + beltuse + POVSTALB + marital + RACE3 + SREGION, 
                    data=a, distance=a$PS2, method="nearest", exact="MALE", discard="both")
    nn2.data <- match.data(nn2)
    summary(nn2, standardize=T)

    # Matching attempt #3
    # Nearest neighbor 1:1 matching using boosted CART estimated PS, 1:1,
    # discarding treated or controls beyond PS overlap, exact matching on sex
    # with caliper of 0.2
    nn3 <- matchit(eversmk ~ LASTAGE + MALE + educate + beltuse + POVSTALB + marital + RACE3 + SREGION, 
                    data=a, distance=a$PS2, method="nearest", exact="MALE", discard="both", caliper=0.2)
    nn3.data <- match.data(nn3)
    summary(nn3, standardize=T)


#~~~ Check balance of 3 matching attempts ~~~#
    # which matching method has the best balance, calculated by ASAM of all covariates (excluding the PS)?
    # Unmatched    
    mean(abs(summary(nn1, standardize=T)$sum.all[, 4][-1]))     # ASAM = 0.103
    
    # Matching attempt #1
    mean(abs(summary(nn1, standardize=T)$sum.matched[, 4][-1])) # ASAM = 0.163

    # Matching attempt #2
    mean(abs(summary(nn2, standardize=T)$sum.matched[, 4][-1])) # ASAM = 0.086

    # Matching attempt #3
    mean(abs(summary(nn3, standardize=T)$sum.matched[, 4][-1])) # ASAM = 0.027

    # Matching attempt #3 was the best

#~~~ Effect estimate ~~~#
# What is the effect of ever smoking on odds of lung cancer / laryngeal cancer / or COPD, as compared with never smoking?
    m1 <- glm(lc5 ~ eversmk, data=nn3.data, family=binomial(link="logit"))
    summary(m1) 
    # OR: 7.25
